from typing import Union

import requests
from tabulate import tabulate

from luscious_dl.logger import logger
from luscious_dl.querys import user_info_query, user_albums_query


class User:
  """User class."""
  def __init__(self, id_: Union[str, int] = None) -> None:
    self.id_ = id_
    self.name = None
    self.number_of_albums = None
    self.albums_ids = []

  def show(self) -> None:
    """Show user information."""
    table = [
      ['ID', self.id_],
      ['Name', self.name],
      ['Albums', self.number_of_albums]
    ]
    logger.log(5, f'User information:\n{tabulate(table, tablefmt="jira")}')

  def fetch_info(self) -> bool:
    """
    Fetch user information.
    :return: bool - true if there are no error otherwise false
    """
    logger.log(5, 'Fetching user information...')
    response = requests.post('https://members.luscious.net/graphql/nobatch/?operationName=ProfileGet',
                             json=user_info_query(str(self.id_))).json()
    data = response['data']['userprofile']['get']
    if "errors" in data:
      logger.error(f'Something wrong with user: {self.id_}\nErrors: {data["errors"]}')
      logger.warning('Skipping...')
      return False
    self.name = data['user']['name']
    self.number_of_albums = data['number_of_albums']
    return True

  def fetch_albums(self) -> None:
    """Fetch user albums."""
    logger.log(5, 'Fetching user albums...')
    n = 1
    raw_data = []
    while True:
      logger.info(f'Fetching user albums page: {n}...')
      response = requests.post('https://members.luscious.net/graphql/nobatch/?operationName=AlbumList',
                               json=user_albums_query(str(self.id_), n)).json()
      raw_data.extend(response['data']['album']['list']['items'])
      n += 1
      if not response['data']['album']['list']['info']['has_next_page']:
        break
    self.albums_ids = [album['id'] for album in raw_data]
    logger.info(f'Total of {len(self.albums_ids)} ids found.')



import os
from argparse import Namespace
from typing import Optional

from luscious_dl.album import Album, search_albums, print_search
from luscious_dl.command_line import command_line
from luscious_dl.downloader import Downloader
from luscious_dl.logger import logger
from luscious_dl.parser import extract_ids_from_list, extract_album_id, extract_user_id
from luscious_dl.user import User
from luscious_dl.utils import info


def albums_download(albums_ids: list[int], downloader: Downloader) -> None:
  """
  Start albums download process.
  :param albums_ids: list of album ids
  :param downloader: Downloder object
  """
  for id_ in albums_ids:
    album = Album(id_)
    try:
      if album.fetch_info():
        album.show()
        album.fetch_pictures()
        album.download(downloader)
      else:
        raise Exception('Album Information not found.')
    except Exception as e:
      logger.critical(f'Album: {id_} Error: {e}')


def users_download(users_ids: list[int], downloader: Downloader) -> None:
  """
  Start users download process.
  :param users_ids: list of user ids
  :param downloader: Downloder object
  """
  for id_ in users_ids:
    user = User(id_)
    try:
      if user.fetch_info():
        user.fetch_albums()
        user.show()
        albums_download(user.albums_ids, downloader)
      else:
        raise Exception('User Information not found.')
    except Exception as e:
      logger.critical(f'User: {id_} Error: {e}')


def normalize_args(args: Namespace) -> Namespace:
  if args.threads <= 0:
    args.threads = os.cpu_count()
  if args.page <= 0:
    args.page = 1
  if args.max_pages <= 0:
    args.max_pages = 1
  if args.page > args.max_pages:
    args.max_pages = args.page

  args.keyword = args.keyword.strip() if args.keyword else None

  if args.album_inputs:
    inputs = [input_.strip() for input_ in args.album_inputs.split(',')]
    args.albums_ids = extract_ids_from_list(inputs, extract_album_id)
  else:
    args.albums_ids = None

  if args.user_inputs:
    inputs = [input_.strip() for input_ in args.user_inputs.split(',')]
    args.users_ids = extract_ids_from_list(inputs, extract_user_id)
  else:
    args.users_ids = None

  return args


def start(args: Namespace = None) -> None:
  """Start"""
  if not args:
    info()
  args = normalize_args(args or command_line())
  downloader = Downloader(args.directory, args.threads, args.retries, args.timeout, args.delay)

  if args.albums_ids:
    albums_download(args.albums_ids, downloader)
  elif args.users_ids:
    users_download(args.users_ids, downloader)
  elif args.keyword:
    result = search_albums(search_query=args.keyword, page=args.page, max_pages=args.max_pages)
    print_search(result)
    if args.search_download:
      for album in result:
        album.show()
        album.fetch_pictures()
        album.download(downloader)


if __name__ == '__main__':
  start()



import json
import os
from typing import Any

from luscious_dl import __version__
from luscious_dl.logger import logger


def info() -> None:
  """Show package version"""
  logger.info(f'Luscious Downloader version: {__version__}')


def cls() -> None:
  """Clears the command prompt"""
  os.system('cls' if os.name == 'nt' else 'clear')


"""def format_filename(name: str) -> str:
  pass"""


def get_config_setting(setting: str) -> Any:
  """
  Retrieve the key value
  :param setting: json key
  :return: key value
  """
  with open('./config.json') as config:
    data = json.load(config)
  return data[setting]


def read_list() -> list[str]:
  """
  Read text file content.
  :return: list content
  """
  try:
    logger.log(5, 'Reading list...')
    with open('./list.txt') as file:
      list_txt = file.read().split('\n')
      logger.log(5, f'Total of Items: {len(list_txt)}.')
    return list_txt
  except Exception as e:
    print(f'Failed to read the list.txt.\n{e}')


def create_default_files() -> None:
  """Create the initial files when using the menu"""
  if not os.path.exists('./config.json'):
    data = {
      "directory": "./Albums/",
      "pool": os.cpu_count(),
      "retries": 5,
      "timeout": 30,
      "delay": 0
    }
    with open('./config.json', 'a+') as config_file:
      json.dump(data, config_file, indent=2)
  if not os.path.exists('./list.xt'):
    open('./list.txt', 'a+')
  if not os.path.exists('./list_completed.txt'):
    open('./list_completed.txt', 'a+')


def create_folder(directory: str) -> None:
  """
  Creates folder in the specified path.
  :param directory: folder path
  """
  try:
    if not os.path.exists(directory):
      os.makedirs(directory, exist_ok=True)
      logger.info(f'Album folder created in: {directory}')
    else:
      logger.warn(f'Album folder already exist in: {directory}')
  except OSError:
    logger.error(f'Creating directory in: {directory}')


class ListOrganizer:
  @staticmethod
  def add(string: str) -> None:
    """
    Add string to list_completed.txt
    :param string: Mostly URL or ID of Album or User
    """
    with open('./list_completed.txt') as completed:
      text = completed.read()
    with open('./list_completed.txt', 'a') as completed:
      if not text.endswith('\n'):
        completed.write('\n')
      completed.write(string)
      logger.log(5, f'Added to completed list: {string}')

  @staticmethod
  def remove(string: str) -> None:
    """
    Remove string from list.txt
    :param string: Mostly URL or ID of Album or User
    """
    with open('./list.txt') as list_txt:
      temp = ['' if string in line else line for line in list_txt]
    with open('./list.txt', 'w') as list_txt:
      for line in temp:
        list_txt.write(line)


def open_config_menu() -> None:
  """Open config.json menu"""
  with open('./config.json', 'r+') as j:
    data = json.load(j)
    while True:
      config_menu = input(f'1 - Change Directory [Current: {data["directory"]}]\n'
                          f'2 - CPU Pool [Current: {data["pool"]}]\n'
                          f'3 - Picture Retries [Current: {data["retries"]}]\n'
                          f'4 - Picture Timeout [Current: {data["timeout"]}]\n'
                          f'5 - Download Delay [Current: {data["delay"]}]\n'
                          '0 - Back and Save.\n'
                          '> ')
      cls()
      if config_menu == '1':
        new_path = input(f'Current directory: {data["directory"]}\n'
                         '1 - Restore default directory\n'
                         '0 - Back\n'
                         'Directory: ')
        if new_path not in ['0', '1', ' ']:
          data['directory'] = os.path.normcase(new_path)
        elif new_path == '1':
          data['directory'] = './Albums/'
        else:
          pass
      elif config_menu == '2':
        print(f'You have: {os.cpu_count()} cores.')
        data['pool'] = int(input('Enter CPU Pool for download pictures.\n> '))
      elif config_menu == '3':
        data['retries'] = int(input('Enter how many pictures retries.\n> '))
      elif config_menu == '4':
        data['timeout'] = int(input('Enter picture timeout.\n> '))
      elif config_menu == '5':
        data['delay'] = int(input('Enter album download delay.\n> '))
      elif config_menu == '0':
        cls()
        break
      else:
        print('Invalid Option.\n')
    j.seek(0)
    json.dump(data, j, indent=2)
    j.truncate()



from typing import Optional, Union, Callable

from luscious_dl.logger import logger


def is_a_valid_integer(x: Union[str, int]) -> bool:
  """
  Check if it's a valid integer.
  :param x: id in string or int format
  :return: bool
  """
  try:
    if isinstance(int(x), int):
      return True
  except (ValueError, TypeError):
    return False


def extract_album_id(album_url: str) -> Optional[int]:
  """
  Extract id from album url.
  :param album_url: album url
  :return: album id
  """
  try:
    split = 2 if album_url.endswith('/') else 1
    album_id = album_url.rsplit('/', split)[1].rsplit('_', 1)[1]
    if is_a_valid_id(album_id):
      return int(album_id)
    else:
      raise Exception('ValueError')
  except Exception as e:
    logger.critical(f"Couldn't resolve album ID of {album_url}\nError: {e}")
    return None


def extract_user_id(user_url: str) -> Optional[int]:
  """
  Extract id from user url.
  :param user_url: user url
  :return: user id
  """
  try:
    split = 2 if user_url.endswith('/') else 1
    user_id = user_url.rsplit('/', split)[1]
    if is_a_valid_id(user_id):
      return int(user_id)
    else:
      raise Exception('ValueError')
  except Exception as e:
    logger.critical(f"Couldn't resolve user ID of {user_url}\nError: {e}")
    return None


def extract_ids_from_list(iterable: list[Union[str, int]], extractor: Callable[[str], Optional[int]]) -> list[int]:
  """
  Extract ids from list containing urls/ids.
  :param iterable: list containing urls/ids
  :param extractor: extraction function
  :return: A list containing the ids
  """
  return list(filter(None, set(int(item) if is_a_valid_id(item) else extractor(item) for item in iterable)))



import multiprocessing as mp
import os
import re
import time
from itertools import repeat

import requests

from luscious_dl.logger import logger
from luscious_dl.utils import create_folder


class Downloader:
  """Downloader class."""
  def __init__(self, output_dir: str, threads: int = 1, retries: int = 5, timeout: int = 30, delay: int = 0) -> None:
    self.output_dir = output_dir
    self.threads = threads
    self.retries = retries
    self.timeout = timeout
    self.delay = delay

  def download_picture(self, picture_url: str, album_folder: str) -> None:
    """
    Download picture.
    :param picture_url: picture url
    :param album_folder: folder path
    """
    try:
      if picture_url.startswith('//'):
        picture_url = picture_url.replace('//', '', 1)
      picture_name = picture_url.rsplit('/', 1)[1]
      picture_path = os.path.join(album_folder, picture_name)
      if not os.path.exists(picture_path):
        logger.info(f'Start downloading: {picture_url}')
        retry = 1
        response = requests.get(picture_url, stream=True, timeout=self.timeout)
        while response.status_code != 200 and retry <= self.retries:
          logger.warning(f'{retry}º Retry: {picture_name}')
          response = requests.get(picture_url, stream=True, timeout=self.timeout)
          retry += 1
        if len(response.content) > 0:
          with open(picture_path, 'wb') as image:
            image.write(response.content)
            logger.log(5, f'Completed download of: {picture_name}')
        else:
          raise Exception('Zero content')
      else:
        logger.warning(f'Picture already exists: {picture_name} ')
    except Exception as e:
      logger.error(f'Failed to download picture: {picture_url}\n{e}')

  def download(self, album_title: str, urls: list[str]) -> None:
    """
    Start download process.
    :param album_title: Album title
    :param urls: list of image URLs
    """
    start_time = time.time()

    album_name = re.sub(r'[^\w\-_\. ]', '_', album_title)
    album_folder = os.path.join(self.output_dir, album_name)
    create_folder(album_folder)

    pool = mp.Pool(self.threads)
    pool.starmap(self.download_picture, zip(urls, repeat(album_folder)))

    end_time = time.time()

    logger.info(f'Album download completed: {album_title}')
    logger.info(f'Finished in {time.strftime("%H:%M:%S", time.gmtime(end_time - start_time))}')

    if self.delay:
      time.sleep(self.delay)



import logging

from colorlog import ColoredFormatter


def logger_file_handler() -> None:
  """Add file handler"""
  file_format = logging.Formatter('%(asctime)s [%(levelname)s] %(message)s', '%Y-%m-%d %H:%M:%S')
  file_handler = logging.FileHandler('logs.log')
  file_handler.setLevel(logging.ERROR)
  file_handler.setFormatter(file_format)
  logger.addHandler(file_handler)


colored_formatter = ColoredFormatter("%(log_color)s[%(asctime)s] [%(levelname)s] %(message)s",
                                     datefmt='%H:%M:%S',
                                     log_colors={
                                       'MSG': 'purple,bold',
                                       'DEBUG': 'cyan',
                                       'INFO': 'green',
                                       'WARNING': 'yellow',
                                       'ERROR': 'red',
                                       'CRITICAL': 'red',
                                     })

logging.addLevelName(5, 'MSG')
logger = logging.getLogger('luscious_dl')

console_format = logging.Formatter('[%(asctime)s] [%(levelname)s] %(message)s', '%H:%M:%S')
console_handler = logging.StreamHandler()
console_handler.setFormatter(colored_formatter)

logger.addHandler(console_handler)
logger.setLevel(5)


if __name__ == '__main__':
  logger.log(5, 'random msg')
  logger.info('info msg')
  logger.warning('warning msg')
  logger.debug('debug msg')
  logger.error('error msg')
  logger.critical('critical msg')
